function [fobj,x,xvec,fvec] = eigopt_constrained(funname, c, pars)
%
% Copyright by Emre Mengi
% Last Modified on September 19, 2015
%
% Locally convergent method of [1] for the following problem:   
% (1)	maximize_{w in R^d} c^T w		subject to lambda_min(A(w)) <= 0
% Above A(w) is an analytic and Hermitian matrix-valued function of w.
%
% Also works for
% (2)	maximize_{w in R^d} c^T w		subject to lambda_max(A(w)) <= 0
% if pars.gamma is chosen large enough.
%
%
% References:
% [1] - E. Mengi. Support function based algorithm for optimization with eigenvalue constraints. 
%		Submitted to SIOPT.
%
% Input:
%	funname (string)	  : name of the routine computing the smallest eigenvalue (or the largest eigenvalue)  and its gradient
%	c (dx1)				  : vector in R^d as in (1)
%	pars (structure)
%							pars.tol1, pars.tol2 - tolerances for termination (default pars.tol1 = pars.tol2 = 10^-6)
%							pars.gamma - a global upper bound for lambda_max( nabla^2 lambda_min(A(w)) )   (or for lambda_max( nabla^2 lambda_max(A(w)) )) 
%							pars.maxit - maximum number of iterations (default pars.maxit = 5000)
%							pars.x0 (dx1) - initial point
%							pars.isplot - plots the progress of the algorithm if nonzero, do not plot if pars.isplot = 0 (default) 
%							pars.isprint - prints the iterates if nonzero, do not print if pars.isprint = 0 (default)
%							parameters for the function funname must be passed through pars
%
% Output
%	fobj (real scalar)	  : computed maximal value
%	x (real scalar)		  : computed local maximizer of (1) 
%	xvec (dx(total#iter)) : the sequence of iterates for the local maximizer converging to x
%	fvex (dx(total#iter)) : the sequence of function values corresponding to f(xvec)



if (~isfield(pars,'x0'))
	error('provide an initial point pars.x0');
end
	 
x = pars.x0;
n = length(x);

	 
	 
	 
if (isfield(pars,'gamma'))
	gamma = pars.gamma;
else	
	gamma = 2;
end

if (isfield(pars,'tol1'))
	tol1 = pars.tol1;
else
	tol1 = 10^-6;
end

if (isfield(pars,'tol2'))
	tol2 = pars.tol2;
else
	tol2 = 10^-6;
end

if (isfield(pars,'maxit'))
	maxit = pars.maxit;
else
	maxit = 5000;
end

if (isfield(pars,'isplot'))
	isplot = pars.isplot;
else
	isplot = 0;
end


if (isfield(pars,'isprint'))
	isprint = pars.isprint;
else
	isprint = 0;
end

iter = 1;
nc = norm(c);
xvec = [x];
fvec = [c'*x];
		

if (isplot)
	figure(1);
	hold on
	plot(x(1),x(2),'bx');
	pause(0.5);
end
		

% this is needed to form the projected Hessian
% V is a basis for orthogonal complement of c
Dummy = [c randn(n,n-1)];
[Q,R] = qr(Dummy);
V = Q(:,2:n);

while ((iter == 1) | (mineig < 0))
	
	[f,g] = feval(funname,x,pars);
		
		
	while ((abs(c'*g/(norm(c)*norm(g))-1) > tol1) | (abs(f) > tol2))

		mup = nc/sqrt(norm(g)^2 - 2*gamma*f);
		x = x + (1/gamma)*((1/mup)*c - g);
	
		xvec = [xvec x];
		fvec = [fvec c'*x];
			
		if ((isplot) & (mod(iter,10) == 0))
			figure(1);
			hold on
			plot(x(1),x(2),'bx');
			pause(0.5);
		end
				
				
		[f,g] = feval(funname,x,pars);
		iter = iter + 1;
		if (isprint)
			fprintf('iter:%d objective:%.8f funval:%.8f\n',iter,c'*x,f);
		end

				
				
		if (iter == maxit)
			fobj = c'*x;
			return;
		end
		

	end
				
				
	H = compute_Hessian(funname,x,pars);
	PH = V'*H*V; % projected Hessian
				

	[PV,PD] = eig(PH);
	PD = diag(PD);
	[mineig,i] = min(PD);
				
	if (mineig < 0)
		if (isprint)
			fprintf('A first order optimal point, but not a local maximizer, is converged.\n'); 
		end
		p = V*PV(:,i); % descent direction
		xm = x + p;
		alpha = 1;
		[f,g] = feval(funname,xm,pars);
				
		while (f > 0)
			alpha = alpha/2;
			xm = x + alpha*p;
			[f,g] = feval(funname,xm,pars);
		end
		
		x = xm;
		xvec = [xvec x];
		fvec = [fvec c'*x];
		if (isprint)
			fprintf('*** restarting ***\n');
		end
	end
		
end
			
fobj = c'*x;

return;